from django.conf import settings


def pytest_configure():
    settings.configure(
        ROOT_URLCONF='tests.urls',

        ALLOWED_HOSTS=['testserver'],

        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': 'test_db'
            }
        },

        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',

            'rest_framework',
            'pushy',
            'tests'
        ],

        PUSHY_GCM_API_KEY='SOME_TEST_KEY',
        PUSHY_GCM_JSON_PAYLOAD=False,
        PUSHY_APNS_CERTIFICATE_FILE='/var/apns/certificate',
        PUSHY_APNS_SANDBOX=False
    )



from rest_framework import viewsets
from rest_framework import status
from rest_framework.response import Response
from rest_framework.settings import api_settings

from pushy.models import Device

from .serializers import DeviceSerializer


class DeviceViewSet(viewsets.ViewSet):
    def create(self, request):
        serializer = DeviceSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save(user_id=request.user.id)

        return Response(
            request.data,
            status=status.HTTP_201_CREATED
        )

    def destroy(self, request):
        try:
            key = request.data.get('key', None)
            Device.objects.get(key=key).delete()

            return Response(
                request.data,
                status=status.HTTP_200_OK
            )
        except Device.DoesNotExist:
            return self._not_found_response(key)

    def _not_found_response(self, key):
        errors_key = api_settings.NON_FIELD_ERRORS_KEY

        return Response(data={
            errors_key: ['Key {} was not found'.format(key)]
        }, status=status.HTTP_404_NOT_FOUND)



import copy
import json
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class PushNotification(models.Model):
    PUSH_INACTIVE = 0
    PUSH_ACTIVE = 1

    PUSH_NOT_SENT = 0
    PUSH_SENT = 1
    PUSH_IN_PROGRESS = 2

    PUSH_CHOICES = (
        (PUSH_ACTIVE, _('Active')),
        (PUSH_INACTIVE, _('Inactive'))
    )

    PUSH_SENT_CHOICES = (
        (PUSH_NOT_SENT, _('Not Sent')),
        (PUSH_IN_PROGRESS, _('In Progress')),
        (PUSH_SENT, _('Sent'))
    )

    title = models.CharField(max_length=50)
    body = models.TextField()

    active = models.SmallIntegerField(choices=PUSH_CHOICES,
                                      default=PUSH_ACTIVE)
    sent = models.SmallIntegerField(choices=PUSH_SENT_CHOICES,
                                    default=PUSH_NOT_SENT)

    date_created = models.DateTimeField(auto_now_add=True)
    date_started = models.DateTimeField(null=True)
    date_finished = models.DateTimeField(null=True)
    filter_type = models.SmallIntegerField(blank=True, default=0)
    filter_user = models.IntegerField(blank=True, default=0)

    @property
    def payload(self):
        if self.body:
            return json.loads(self.body)
        return None

    @payload.setter
    def payload(self, value):
        self.body = json.dumps(value)

    def to_dict(self):
        # return notification as a dictionary
        # not including duplicate or model related fields
        data = copy.deepcopy(self.__dict__)
        data['payload'] = self.payload
        del data['body']
        del data['_state']
        return data

    def __unicode__(self):
        return self.title


class Device(models.Model):
    DEVICE_TYPE_ANDROID = 1
    DEVICE_TYPE_IOS = 2
    DEVICE_TYPE_CHOICES = (
        (DEVICE_TYPE_ANDROID, 'Android'),
        (DEVICE_TYPE_IOS, 'iOS')
    )

    key = models.CharField(max_length=255)
    type = models.SmallIntegerField(choices=DEVICE_TYPE_CHOICES)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)

    class Meta:
        unique_together = ('key', 'type')

    def __unicode__(self):
        # Reverse choices
        device_choices = dict(reversed(self.DEVICE_TYPE_CHOICES))
        return "{} Device ID: {}".format(device_choices[self.type], self.pk)


def get_filtered_devices_queryset(notification):
    devices = Device.objects.all()

    if 'filter_type' in notification and notification['filter_type']:
        devices = devices.filter(type=notification['filter_type'])
    if 'filter_user' in notification and notification['filter_user']:
        devices = devices.filter(user_id=notification['filter_user'])

    return devices



import mock

from django.test import TestCase

from pushjack.apns import APNSSandboxClient
from pushjack.exceptions import (
    GCMMissingRegistrationError,
    GCMInvalidPackageNameError,
    GCMTimeoutError,
    GCMAuthError,
    APNSAuthError,
    APNSMissingTokenError,
    APNSProcessingError,
    APNSShutdownError
)

from pushy.exceptions import (
    PushInvalidTokenException,
    PushInvalidDataException,
    PushAuthException,
    PushServerException
)

from pushy.models import Device
from pushy import dispatchers

from .data import (
    valid_response,
    valid_with_canonical_id_response,
    invalid_with_exception
)


class DispatchersTestCase(TestCase):

    def test_check_cache(self):
        dispatchers.dispatchers_cache = {}

        # Test cache Android
        dispatcher1 = dispatchers.get_dispatcher(Device.DEVICE_TYPE_ANDROID)
        self.assertEquals(dispatchers.dispatchers_cache, {1: dispatcher1})

        # Test cache iOS
        dispatcher2 = dispatchers.get_dispatcher(Device.DEVICE_TYPE_IOS)
        self.assertEquals(
            dispatchers.dispatchers_cache,
            {1: dispatcher1, 2: dispatcher2}
        )

        # Final check, fetching from cache
        dispatcher1 = dispatchers.get_dispatcher(Device.DEVICE_TYPE_ANDROID)
        dispatcher2 = dispatchers.get_dispatcher(Device.DEVICE_TYPE_IOS)
        self.assertEquals(
            dispatchers.dispatchers_cache,
            {1: dispatcher1, 2: dispatcher2}
        )

    def test_dispatcher_types(self):
        # Double check the factory method returning the correct types
        self.assertIsInstance(
            dispatchers.get_dispatcher(Device.DEVICE_TYPE_ANDROID),
            dispatchers.GCMDispatcher
        )
        self.assertIsInstance(
            dispatchers.get_dispatcher(Device.DEVICE_TYPE_IOS),
            dispatchers.APNSDispatcher
        )


class GCMDispatcherTestCase(TestCase):
    device_key = 'TEST_DEVICE_KEY'
    data = {'title': 'Test', 'body': 'Test body'}

    def test_constructor_with_api_key(self):
        dispatcher = dispatchers.GCMDispatcher(123)
        self.assertEquals(123, dispatcher._api_key)

    def test_send_with_no_api_key(self):
        # Check that we throw the proper exception
        # in case no API Key is specified
        with mock.patch('django.conf.settings.PUSHY_GCM_API_KEY', new=None):
            dispatcher = dispatchers.GCMDispatcher()
            self.assertRaises(
                PushAuthException,
                dispatcher.send,
                self.device_key,
                self.data
            )

    def test_notification_sent(self):
        dispatcher = dispatchers.GCMDispatcher()
        with mock.patch('pushjack.GCMClient.send') as request_mock:
            request_mock.return_value = valid_response()
            dispatcher.send(self.device_key, self.data)
            self.assertTrue(request_mock.called)

    def test_notification_sent_with_canonical_id(self):
        dispatcher = dispatchers.GCMDispatcher()
        # Check result when canonical value is returned
        response_mock = mock.Mock()
        response_mock.return_value = valid_with_canonical_id_response(123123)
        with mock.patch('pushjack.GCMClient.send', new=response_mock):
            canonical_id = dispatcher.send(self.device_key, self.data)
            self.assertEquals(canonical_id, 123123)

    def test_invalid_token_exception(self):
        dispatcher = dispatchers.GCMDispatcher()
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            GCMAuthError('')
        )
        with mock.patch('pushjack.GCMClient.send', new=response_mock):
            self.assertRaises(
                PushAuthException,
                dispatcher.send,
                self.device_key,
                self.data
            )

    def test_invalid_api_key_exception(self):
        dispatcher = dispatchers.GCMDispatcher()
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            GCMMissingRegistrationError('')
        )
        with mock.patch('pushjack.GCMClient.send', new=response_mock):
            self.assertRaises(
                PushInvalidTokenException,
                dispatcher.send,
                self.device_key,
                self.data
            )

    def test_invalid_data_exception(self):
        dispatcher = dispatchers.GCMDispatcher()
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            GCMInvalidPackageNameError('')
        )
        with mock.patch('pushjack.GCMClient.send', new=response_mock):
            self.assertRaises(
                PushInvalidDataException,
                dispatcher.send,
                self.device_key,
                self.data
            )

    def test_invalid_exception(self):
        dispatcher = dispatchers.GCMDispatcher()
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            GCMTimeoutError('')
        )
        with mock.patch('pushjack.GCMClient.send', new=response_mock):
            self.assertRaises(
                PushServerException,
                dispatcher.send,
                self.device_key,
                self.data
            )


class ApnsDispatcherTests(TestCase):
    device_key = 'TEST_DEVICE_KEY'
    data = {'alert': 'Test'}

    def setUp(self):
        self.dispatcher = dispatchers.get_dispatcher(Device.DEVICE_TYPE_IOS)

    @mock.patch('django.conf.settings.PUSHY_APNS_CERTIFICATE_FILE', new=None)
    def test_certificate_exception_on_send(self):
        self.assertRaises(
            PushAuthException,
            self.dispatcher.send,
            self.device_key,
            self.data
        )

    @mock.patch('django.conf.settings.PUSHY_APNS_SANDBOX', new=True)
    def test_sandbox_client(self):
        dispatcher = dispatchers.APNSDispatcher()
        dispatcher.establish_connection()
        self.assertIsInstance(dispatcher._client, APNSSandboxClient)

    def test_invalid_token_exception(self):
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            APNSMissingTokenError('')
        )
        with mock.patch('pushjack.APNSClient.send', new=response_mock):
            self.assertRaises(
                PushInvalidTokenException,
                self.dispatcher.send,
                self.device_key,
                self.data
            )

    def test_invalid_api_key_exception(self):
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            APNSAuthError('')
        )
        with mock.patch('pushjack.APNSClient.send', new=response_mock):
            self.assertRaises(
                PushAuthException,
                self.dispatcher.send,
                self.device_key,
                self.data
            )

    def test_invalid_data_exception(self):
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            APNSProcessingError('')
        )
        with mock.patch('pushjack.APNSClient.send', new=response_mock):
            self.assertRaises(
                PushInvalidDataException,
                self.dispatcher.send,
                self.device_key,
                self.data
            )

    def test_invalid_exception(self):
        # Check not registered exception
        response_mock = mock.Mock()
        response_mock.return_value = invalid_with_exception(
            APNSShutdownError('')
        )
        with mock.patch('pushjack.APNSClient.send', new=response_mock):
            self.assertRaises(
                PushServerException,
                self.dispatcher.send,
                self.device_key,
                self.data
            )

    def test_push_sent(self):
        apns = mock.Mock()
        apns.return_value = valid_response()
        with mock.patch('pushjack.APNSClient.send', new=apns):
            self.assertEqual(
                self.dispatcher.send(self.device_key, self.data),
                None
            )



import datetime
import mock

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.utils import override_settings
from django.utils import timezone

from pushy.models import (
    PushNotification,
    Device,
    get_filtered_devices_queryset
)

from pushy.exceptions import (
    PushException,
    PushInvalidTokenException
)

from pushy.tasks import (
    check_pending_push_notifications,
    send_push_notification_group,
    send_single_push_notification,
    create_push_notification_groups,
    clean_sent_notifications,
    notify_push_notification_sent
)


class TasksTestCase(TestCase):
    def setUp(self):
        self.payload = {
            'key1': 'value1',
            'key2': 'value2',
        }

    def test_get_filtered_devices_queryset_on_type(self):
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT,
            filter_type=Device.DEVICE_TYPE_IOS
        )

        for i in range(0, 3):
            Device.objects.create(
                key='TEST_DEVICE_KEY_{}'.format(i),
                type=Device.DEVICE_TYPE_IOS
            )

        for i in range(0, 10):
            Device.objects.create(
                key='TEST_DEVICE_KEY_{}'.format(i),
                type=Device.DEVICE_TYPE_ANDROID
            )

        devices = get_filtered_devices_queryset(notification.to_dict())
        self.assertEqual(devices.count(), 3)

    def test_get_filtered_devices_queryset_on_user(self):
        for i in range(0, 3):
            get_user_model().objects.create_user(
                username='test_user_%d' % i,
                email='test_user_%d@django-pushy.com' % i,
                password=i
            )

        user1 = get_user_model().objects.get(pk=1)
        user2 = get_user_model().objects.get(pk=2)

        # Add 5 devices to user2
        for i in range(0, 5):
            Device.objects.create(
                key='TEST_DEVICE_KEY_{}'.format(i),
                type=Device.DEVICE_TYPE_ANDROID,
                user=user2
            )

        # Check that user2 has 5 devices
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT,
            filter_user=user2.id
        )

        devices = get_filtered_devices_queryset(notification.to_dict())

        self.assertEqual(devices.count(), 5)

        # Check that user 1 has no devices
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT,
            filter_user=user1.id
        )

        devices = get_filtered_devices_queryset(notification.to_dict())

        self.assertEqual(devices.count(), 0)

    def test_pending_notifications(self):
        PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )

        mocked_task = mock.Mock()
        with mock.patch(
                'pushy.tasks.create_push_notification_groups.apply_async',
                new=mocked_task):
            check_pending_push_notifications()
            mocked_task.assert_called()

    def test_notifications_groups_chord(self):
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )

        # Create a test device key
        Device.objects.create(
            key='TEST_DEVICE_KEY_ANDROID',
            type=Device.DEVICE_TYPE_ANDROID
        )

        mocked_task = mock.Mock()
        with mock.patch(
                'celery.chord',
                new=mocked_task):
            create_push_notification_groups(notification.to_dict())
            mocked_task.assert_called()

    def test_notifications_groups_return(self):
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )

        # Create a test device key
        Device.objects.create(
            key='TEST_DEVICE_KEY_ANDROID',
            type=Device.DEVICE_TYPE_ANDROID
        )

        mocked_task = mock.Mock()
        with mock.patch(
                'celery.chord',
                new=mocked_task):
            notification_dict = notification.to_dict()
            notification_dict['id'] = None
            create_push_notification_groups(notification_dict)

            notification = PushNotification.objects.get(pk=notification.id)
            self.assertEqual(PushNotification.PUSH_NOT_SENT, notification.sent)

    def test_send_notification_groups(self):
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )

        # Create a test device key
        device = Device.objects.create(
            key='TEST_DEVICE_KEY_ANDROID',
            type=Device.DEVICE_TYPE_ANDROID
        )

        # Make sure canonical ID is saved
        gcm = mock.Mock()
        gcm.return_value = 123123
        with mock.patch('pushy.dispatchers.GCMDispatcher.send', new=gcm):
            send_push_notification_group(notification.to_dict(), 0, 1)

            device = Device.objects.get(pk=device.id)
            self.assertEqual(device.key, '123123')

        # Make sure the key is deleted when not registered exception is fired
        gcm = mock.Mock()
        gcm.side_effect = PushInvalidTokenException
        with mock.patch('pushy.dispatchers.GCMDispatcher.send', new=gcm):
            send_push_notification_group(notification.to_dict(), 0, 1)

            self.assertRaises(
                Device.DoesNotExist,
                Device.objects.get,
                pk=device.id
            )

        # Create an another test device key
        device = Device.objects.create(
            key='TEST_DEVICE_KEY_ANDROID2',
            type=Device.DEVICE_TYPE_ANDROID
        )

        # No canonical ID wasn't returned
        gcm = mock.Mock()
        gcm.return_value = False
        with mock.patch('pushy.dispatchers.GCMDispatcher.send', new=gcm):
            send_push_notification_group(notification.to_dict(), 0, 1)

            device = Device.objects.get(pk=device.id)
            self.assertEqual(device.key, 'TEST_DEVICE_KEY_ANDROID2')

    def test_delete_old_key_if_canonical_is_registered(self):
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )
        # Create a test device key
        device = Device.objects.create(
            key='TEST_DEVICE_KEY_ANDROID',
            type=Device.DEVICE_TYPE_ANDROID
        )
        Device.objects.create(key='123123', type=Device.DEVICE_TYPE_ANDROID)

        # Make sure old device is deleted
        # if the new canonical ID already exists
        gcm = mock.Mock()
        gcm.return_value = '123123'
        with mock.patch('pushy.dispatchers.GCMDispatcher.send', new=gcm):
            send_push_notification_group(notification.to_dict(), 0, 1)

            self.assertFalse(Device.objects.filter(pk=device.id).exists())

    def test_create_push_notification_groups_non_existent_notification(self):
        result = create_push_notification_groups({'id': 1000})
        self.assertFalse(result)

    def test_non_existent_device(self):
        result = send_single_push_notification(1000, {'payload': 'test'})
        self.assertFalse(result)

    def test_invalid_token_exception(self):
        # Create a test device key
        device = Device.objects.create(
            key='TEST_DEVICE_KEY_ANDROID',
            type=Device.DEVICE_TYPE_ANDROID
        )
        gcm = mock.Mock()
        gcm.side_effect = PushInvalidTokenException
        with mock.patch('pushy.dispatchers.GCMDispatcher.send', new=gcm):
            send_single_push_notification(device, {'payload': 'test'})
            self.assertRaises(
                Device.DoesNotExist,
                Device.objects.get,
                pk=device.id
            )

    @mock.patch('pushy.tasks.logger.exception')
    def test_push_exception(self, logging_mock):
        # Create a test device key
        device = Device.objects.create(
            key='TEST_DEVICE_KEY_ANDROID',
            type=Device.DEVICE_TYPE_ANDROID
        )
        gcm = mock.Mock()
        gcm.side_effect = PushException
        with mock.patch('pushy.dispatchers.GCMDispatcher.send', new=gcm):
            send_single_push_notification(device, {'payload': 'test'})

            logging_mock.assert_called()

    def test_delete_old_notifications_undefine_max_age(self):
        self.assertRaises(ValueError, clean_sent_notifications)

    @override_settings(PUSHY_NOTIFICATION_MAX_AGE=datetime.timedelta(days=90))
    def test_delete_old_notifications(self):
        for i in range(10):
            date_started = timezone.now() - datetime.timedelta(days=91)
            date_finished = date_started
            notification = PushNotification()
            notification.title = 'title {}'.format(i)
            notification.body = '{}'
            notification.sent = PushNotification.PUSH_SENT
            notification.date_started = date_started
            notification.date_finished = date_finished
            notification.save()

        clean_sent_notifications()

        self.assertEquals(PushNotification.objects.count(), 0)

    @override_settings(PUSHY_NOTIFICATION_MAX_AGE=datetime.timedelta(days=90))
    def test_delete_old_notifications_with_remaining_onces(self):
        for i in range(10):
            date_started = timezone.now() - datetime.timedelta(days=91)
            date_finished = date_started
            notification = PushNotification()
            notification.title = 'title {}'.format(i)
            notification.body = '{}'
            notification.sent = PushNotification.PUSH_SENT
            notification.date_started = date_started
            notification.date_finished = date_finished
            notification.save()

        for i in range(10):
            date_started = timezone.now() - datetime.timedelta(days=61)
            date_finished = date_started
            notification = PushNotification()
            notification.title = 'title {}'.format(i)
            notification.body = '{}'
            notification.sent = PushNotification.PUSH_SENT
            notification.date_started = date_started
            notification.date_finished = date_finished
            notification.save()

        clean_sent_notifications()

        self.assertEquals(PushNotification.objects.count(), 10)

    def test_notify_notification_finished(self):
        notification = PushNotification.objects.create(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )
        notification.save()
        notify_push_notification_sent(notification.to_dict())

        notification = PushNotification.objects.get(pk=notification.id)
        self.assertEquals(PushNotification.PUSH_SENT, notification.sent)

    def test_notify_notification_does_not_exist(self):
        notification = PushNotification(
            id=1001,
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )

        notify_push_notification_sent(notification.to_dict())
        self.assertEquals(PushNotification.PUSH_NOT_SENT, notification.sent)

    def test_notify_notification_does_nothing(self):
        notification = PushNotification(
            title='test',
            payload=self.payload,
            active=PushNotification.PUSH_ACTIVE,
            sent=PushNotification.PUSH_NOT_SENT
        )

        self.assertFalse(notify_push_notification_sent(notification.to_dict()))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('key', models.TextField()),
                ('type', models.SmallIntegerField(choices=[(1, 'Android'), (2, 'iOS')])),
                ('user', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PushNotification',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=50)),
                ('body', models.TextField()),
                ('active', models.SmallIntegerField(default=1, choices=[(1, 'Active'), (0, 'Inactive')])),
                ('sent', models.SmallIntegerField(default=0, choices=[(0, 'Not Sent'), (1, 'Sent')])),
                ('date_created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.core.urlresolvers import reverse
from rest_framework import status
from rest_framework.test import APITestCase


class APITests(APITestCase):
    def setUp(self):
        self.email = 'test_user@some_domain.com'
        self.password = 'test_password'

        # self.user = User.objects.create_user(self.email, self.password)

    def test_create_device(self):
        data = {'key': 'KEY1', 'type': 'android'}

        response = self.create_device(data)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data, data)

    def test_create_device_ios(self):
        data = {'key': 'KEY1', 'type': 'ios'}

        response = self.create_device(data)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data, data)

    def test_create_device_unknown_type(self):
        data = {'key': 'KEY1', 'type': 'unknown'}

        response = self.create_device(data)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {'type': ['"unknown" is not a valid choice.']})

    def test_create_duplicate_device(self):
        data = {'key': 'KEY1', 'type': 'android'}

        self.create_device(data)

        response = self.create_device(data)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['non_field_errors'][0],
                         'The fields key, type must make a unique set.')

    def test_destroy_device(self):
        data = {'key': 'KEY1', 'type': 'ios'}
        self.create_device(data)
        response = self.destroy_device(data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_destroy_nonexistent_device(self):
        response = self.destroy_device({'key': 'does not exist'})
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def create_device(self, data):
        url = reverse('pushy-devices')

        response = self.client.post(url, data, format='json')
        return response

    def destroy_device(self, data):
        url = reverse('pushy-devices')

        response = self.client.delete(url, data, format='json')
        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pushy', '0002_auto_20141011_1703'),
    ]

    operations = [
        migrations.AddField(
            model_name='pushnotification',
            name='date_finished',
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name='pushnotification',
            name='date_started',
            field=models.DateTimeField(null=True),
        ),
    ]



from .models import PushNotification
from .tasks import (
    send_single_push_notification,
    create_push_notification_groups
)


def send_push_notification(title, payload, device=None,
                           filter_user=None, filter_type=None,
                           store=True):

    if not filter_type:
        filter_type = 0
    if not filter_user:
        filter_user = 0

    notification = PushNotification(
        title=title,
        payload=payload,
        active=PushNotification.PUSH_ACTIVE,
        sent=PushNotification.PUSH_NOT_SENT,
        filter_user=filter_user,
        filter_type=filter_type
    )
    if store:
        notification.save()

    if device:
        # Send a single push notification immediately
        send_single_push_notification.apply_async(kwargs={
            'device': device.id,
            'notification': notification.to_dict()
        })
        return notification

    create_push_notification_groups.delay(notification=notification.to_dict())

    return notification



# assert warnings are enabled
import warnings

warnings.simplefilter('ignore', Warning)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'pushy',
]

MIDDLEWARE_CLASSES = []

SECRET_KEY = 'django-pushy-key'
SITE_ID = 1
ROOT_URLCONF = 'core.urls'

CELERY_ALWAYS_EAGER = True
PUSHY_GCM_API_KEY = 'blah-blah'

PUSHY_GCM_JSON_PAYLOAD = False

PUSHY_APNS_CERTIFICATE_FILE = 'aps_development.pem'
PUSHY_APNS_KEY_FILE = 'private_key.pem'


import copy
from django.conf import settings
from pushjack import (
    APNSClient,
    APNSSandboxClient,
    GCMClient
)
from pushjack.exceptions import (
    GCMAuthError,
    GCMMissingRegistrationError,
    GCMInvalidRegistrationError,
    GCMUnregisteredDeviceError,
    GCMInvalidPackageNameError,
    GCMMismatchedSenderError,
    GCMMessageTooBigError,
    GCMInvalidDataKeyError,
    GCMInvalidTimeToLiveError,
    GCMTimeoutError,
    GCMInternalServerError,
    GCMDeviceMessageRateExceededError,

    APNSAuthError,
    APNSProcessingError,
    APNSMissingTokenError,
    APNSMissingTopicError,
    APNSMissingPayloadError,
    APNSInvalidTokenSizeError,
    APNSInvalidTopicSizeError,
    APNSInvalidPayloadSizeError,
    APNSInvalidTokenError,
    APNSShutdownError,
    APNSUnknownError
)

from .models import Device
from .exceptions import (
    PushAuthException,
    PushInvalidTokenException,
    PushInvalidDataException,
    PushServerException
)

dispatchers_cache = {}


class Dispatcher(object):
    def send(self, device_key, data):
        raise NotImplementedError()


class APNSDispatcher(Dispatcher):
    def __init__(self):
        super(APNSDispatcher, self).__init__()
        self._client = None

    @property
    def cert_file(self):
        return getattr(settings, 'PUSHY_APNS_CERTIFICATE_FILE', None)

    @property
    def use_sandbox(self):
        return bool(getattr(settings, 'PUSHY_APNS_SANDBOX', False))

    def establish_connection(self):
        if self.cert_file is None:
            raise PushAuthException('Missing APNS certificate error')

        target_class = APNSClient
        if self.use_sandbox:
            target_class = APNSSandboxClient

        self._client = target_class(
            certificate=self.cert_file,
            default_error_timeout=10,
            default_expiration_offset=2592000,
            default_batch_size=100
        )

    def _send(self, token, notification_payload):
        # pop causes a bug in altering the original payload
        # which causes title and message to be empty
        # for notifications following the currrent one.
        payload = copy.deepcopy(notification_payload)

        try:
            response = self._client.send(
                [token],
                title=payload.pop('title', None),
                message=payload.pop('message', None),
                sound=payload.pop('sound', None),
                badge=payload.pop('badge', None),
                category=payload.pop('category', None),
                content_available=True,
                extra=payload or {}
            )

            if response.errors:
                raise response.errors.pop()
            return None

        except APNSAuthError:
            raise PushAuthException()

        except (APNSMissingTokenError,
                APNSInvalidTokenError):
            raise PushInvalidTokenException()

        except (APNSProcessingError,
                APNSMissingTopicError,
                APNSMissingPayloadError,
                APNSInvalidTokenSizeError,
                APNSInvalidTopicSizeError,
                APNSInvalidPayloadSizeError):
            raise PushInvalidDataException()

        except (APNSShutdownError,
                APNSUnknownError):
            raise PushServerException()

        except:
            raise PushServerException()

    def send(self, device_key, payload):
        if not self._client:
            self.establish_connection()

        return self._send(device_key, payload)


class GCMDispatcher(Dispatcher):
    def __init__(self, api_key=None):
        if not api_key:
            api_key = getattr(settings, 'PUSHY_GCM_API_KEY', None)
        self._api_key = api_key

    def _send(self, device_key, payload):
        if not self._api_key:
            raise PushAuthException()

        gcm_client = GCMClient(self._api_key)
        try:
            response = gcm_client.send(
                [device_key],
                payload
            )

            if response.errors:
                raise response.errors.pop()

            canonical_id = None
            if response.canonical_ids:
                canonical_id = response.canonical_ids[0].new_id
            return canonical_id

        except GCMAuthError:
            raise PushAuthException()

        except (GCMMissingRegistrationError,
                GCMInvalidRegistrationError,
                GCMUnregisteredDeviceError):
            raise PushInvalidTokenException()

        except (GCMInvalidPackageNameError,
                GCMMismatchedSenderError,
                GCMMessageTooBigError,
                GCMInvalidDataKeyError,
                GCMInvalidTimeToLiveError):
            raise PushInvalidDataException()

        except (GCMTimeoutError,
                GCMInternalServerError,
                GCMDeviceMessageRateExceededError):
            raise PushServerException()

        except:
            raise PushServerException()

    def send(self, device_key, payload):
        return self._send(device_key, payload)


def get_dispatcher(device_type):
    if device_type in dispatchers_cache and dispatchers_cache[device_type]:
        return dispatchers_cache[device_type]

    if device_type == Device.DEVICE_TYPE_ANDROID:
        dispatchers_cache[device_type] = GCMDispatcher()
    else:
        dispatchers_cache[device_type] = APNSDispatcher()

    return dispatchers_cache[device_type]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-20 18:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pushy', '0003_auto_20150902_2001'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pushnotification',
            name='sent',
            field=models.SmallIntegerField(choices=[(0, 'Not Sent'), (2, 'In Progress'), (1, 'Sent')], default=0),
        ),
    ]



from django.conf.urls import include, url
from django.contrib import admin

from pushy.contrib.rest_api import urls as rest_urls

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
    url(r'^api/', include(rest_urls))
]



from setuptools import setup

setup(
    name='Django-Pushy',
    version='1.0.2',
    author='Rakan Alhneiti',
    author_email='rakan.alhneiti@gmail.com',

    # Packages
    packages=[
        'pushy',
        'pushy/contrib',
        'pushy/contrib/rest_api',
        'pushy/migrations',
    ],
    include_package_data=True,

    # Details
    url='https://github.com/rakanalh/django-pushy',

    license='LICENSE.txt',
    description='Handle push notifications at scale.',
    long_description=open('README.rst').read(),

    # Dependent packages (distributions)
    install_requires=[
        'django>=1.6',
        'django-celery==3.1.17',
        'pushjack==1.3.0'
    ],
    extras_require={
        'rest_api': ['djangorestframework>=3.0,<3.3']
    }
)



from django.contrib.auth import get_user_model
import mock
from django.test import TestCase
from pushy.utils import send_push_notification
from pushy.models import PushNotification, Device


class AddTaskTestCase(TestCase):
    def setUp(self):
        self.payload = {
            'key1': 'value1',
            'key2': 'value2',
        }

    def test_add_task(self):
        mock_task = mock.Mock()

        with mock.patch('pushy.tasks.create_push_notification_groups.delay',
                        new=mock_task) as mocked_task:
            send_push_notification('some test push notification', self.payload)
            notification = PushNotification.objects.latest('id')
            mocked_task.assert_called_once_with(
                notification=notification.to_dict()
            )

            self.assertEquals(notification.payload, self.payload)

    def test_add_task_filter_device(self):
        device = Device.objects.create(key='TEST_DEVICE_KEY',
                                       type=Device.DEVICE_TYPE_IOS)

        mock_task = mock.Mock()
        with mock.patch('pushy.tasks.send_single_push_notification.apply_async',
                        new=mock_task) as mocked_task:
            send_push_notification(
                'some other test push notification',
                self.payload,
                device=device
            )

            notification = PushNotification.objects.latest('id')

            mocked_task.assert_called_with(kwargs={
                'device': device.id,
                'notification': notification.to_dict()
            })

    def test_add_task_filter_on_user(self):
        user = get_user_model().objects.create_user(
            username='test_user',
            email='test_user@django-pushy.com',
            password='test_password'
        )

        mock_task = mock.Mock()
        with mock.patch('pushy.tasks.create_push_notification_groups.delay',
                        new=mock_task) as mocked_task:
            send_push_notification(
                'some other test push notification',
                self.payload,
                filter_user=user.id
            )

            notification = PushNotification.objects.latest('id')

            mocked_task.assert_called_with(notification=notification.to_dict())
            self.assertEqual(notification.filter_user, user.id)
            self.assertEqual(notification.filter_type, 0)

    def test_add_task_filter_on_device_type(self):
        mock_task = mock.Mock()
        with mock.patch('pushy.tasks.create_push_notification_groups.delay',
                        new=mock_task) as mocked_task:
            send_push_notification(
                'some other test push notification',
                self.payload,
                filter_type=Device.DEVICE_TYPE_IOS
            )

            notification = PushNotification.objects.latest('id')

            mocked_task.assert_called_with(notification=notification.to_dict())
            self.assertEqual(notification.filter_user, 0)
            self.assertEqual(notification.filter_type, Device.DEVICE_TYPE_IOS)

    def test_add_task_filter_on_device_type_and_user(self):
        user = get_user_model().objects.create_user(
            username='test_user',
            email='test_user@django-pushy.com',
            password='test_password'
        )

        mock_task = mock.Mock()
        with mock.patch('pushy.tasks.create_push_notification_groups.delay',
                        new=mock_task) as mocked_task:
            send_push_notification(
                'some other test push notification', self.payload,
                filter_type=Device.DEVICE_TYPE_IOS,
                filter_user=user.id
            )

            notification = PushNotification.objects.latest('id')

            mocked_task.assert_called_with(notification=notification.to_dict())
            self.assertEqual(notification.filter_user, user.id)
            self.assertEqual(notification.filter_type, Device.DEVICE_TYPE_IOS)

    def test_add_task_without_storage(self):
        with mock.patch('pushy.tasks.create_push_notification_groups.delay'):
            notification = send_push_notification(
                'test', {},
                filter_type=Device.DEVICE_TYPE_IOS,
                store=False
            )

            self.assertIsNone(notification.id)
